"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = ODApp;

var _bluebird = _interopRequireWildcard(require("bluebird"));

var _bodyParser = _interopRequireDefault(require("body-parser"));

var _cookieParser = _interopRequireDefault(require("cookie-parser"));

var _debug = _interopRequireDefault(require("debug"));

var _express = _interopRequireDefault(require("express"));

var _http = _interopRequireDefault(require("http"));

var _morgan = _interopRequireDefault(require("morgan"));

var _handler = require("./handler");

var _mysql = require("./mysql");

var _serverContext = require("./server-context");

var _definition = require("./storage/definition");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function ODApp(config = {}) {
  const app = (0, _express.default)();

  const httpServer = _http.default.createServer(app);

  const handlerDefinitions = [];
  const storageDefinitions = [];
  const initializations = [];
  const di = {
    options: Object.assign({
      PORT: 8082,
      MYSQL: {},
      COOKIE_SECRET: 'od-cookie-secret',
      ENABLE_CORS: false,
      MORGAN: '',
      BODY_PARSER_JSON_LIMIT: '1mb',
      STORAGE: {}
    }, config),
    express: app,
    httpServer,
    storage: {} // name - driver pair

  };
  const debug = (0, _debug.default)('od:ODApp');
  const defineHandler = (0, _handler.handlerDefiner)(di.options);
  const defineStorage = (0, _definition.storageDefiner)(di.options.STORAGE);
  return Object.create(null, {
    addInit: {
      writable: false,
      configurable: true,

      value(initCode) {
        initializations.push(() => (0, _bluebird.resolve)(initCode()));
      }

    },
    di: {
      writable: false,
      configurable: true,
      value: di
    },
    defineHandler: {
      writable: false,
      configurable: false,

      value(name, cb) {
        const definer = defineHandler(name);
        cb(definer);
        handlerDefinitions.push(definer.build());
      }

    },
    defineStorage: {
      writable: false,
      configurable: true,

      value(name, cb) {
        const definer = defineStorage(name);
        cb && cb(definer);
        storageDefinitions.push(definer.build());
      }

    },
    run: {
      writable: false,
      configurable: false,

      async value({
        noListen = false
      } = {}) {
        const {
          options
        } = di; //
        // Initialize MySQL
        //

        if (options.MYSQL) {
          di.mysql = await (0, _mysql.initMySQLPool)(options.MYSQL);
        } //
        // Setup express
        //


        app.use((0, _cookieParser.default)(options.COOKIE_SECRET));

        if (options.ENABLE_CORS) {
          app.use(function corsHandler(req, res, next) {
            res.header('Access-Control-Allow-Credentials', true);
            res.header('Access-Control-Allow-Origin', req.headers.origin);
            res.header('Access-Control-Allow-Methods', 'GET,PUT,POST,DELETE');
            res.header('Access-Control-Allow-Headers', 'X-Requested-With, X-HTTP-Method-Override, Content-Type, Accept, DeviceInfo, Authorization');
            next();
          });
        }

        options.MORGAN && app.use((0, _morgan.default)(options.MORGAN));
        app.use(_bodyParser.default.json({
          limit: options.BODY_PARSER_JSON_LIMIT
        }));
        app.use(_bodyParser.default.urlencoded({
          extended: true
        })); //
        // Initialize Storage
        //

        await (0, _bluebird.map)(storageDefinitions, definition => {
          const {
            name,
            driver
          } = definition;
          debug(`Initializing storage driver for ${name}`);
          di.storage[name] = driver;
          return driver.init();
        }); //
        // Initialize Context
        //

        const contextWrapper = (0, _serverContext.ContextWrapper)(options);

        if (this.initServer) {
          await this.initServer();
        }

        handlerDefinitions.forEach(definition => {
          const {
            endpoint: {
              url,
              method
            },
            name
          } = definition;
          debug(`Mounting ${name} for [${method}] ${url}`);
          app[method](url, contextWrapper.wrap(di, definition));
        });

        if (this.afterHandlerMounted) {
          await this.afterHandlerMounted();
        } //
        // Mount final error logger
        //
        // eslint-disable-next-line no-unused-vars


        app.use(function errorHandler(err, req, res, next) {
          di.onError && di.onError(err);

          if (err.handled) {
            // console.log(`Handler returned error : `, err)
            res.status(err.status).json({
              error: {
                message: err.message,
                code: err.code
              }
            });
          } else {
            console.error(err.stack);
            res.status(500).json({
              error: err.message
            });
          }
        }); //
        // Initialization routines
        //

        for (let i = 0; i < initializations.length; i += 1) {
          // eslint-disable-next-line no-await-in-loop
          await initializations[i]();
        } //
        // Start Listening
        //


        if (noListen) {
          return (0, _bluebird.resolve)();
        }

        return new _bluebird.default(resolve => httpServer.listen(options.PORT || 8082, resolve));
      }

    },
    stop: {
      writable: false,
      configurable: false,

      async value() {
        await di.mysql.end();
      }

    }
  });
}