"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handlerDefiner = handlerDefiner;
exports.handlerMaker = handlerMaker;

var _bluebird = require("bluebird");

var _knex = _interopRequireDefault(require("knex"));

var _lodash = _interopRequireDefault(require("lodash"));

var _odJs = require("od-js");

var _mysql = require("./mysql");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const knex = (0, _knex.default)({
  client: 'mysql'
});
const sane = (0, _odJs.sanitizer)();

function handlerDefiner() {
  return function defineHandler(name) {
    const d = {
      name,
      handler: null,
      options: {},
      endpoint: {}
    };
    return Object.create(null, {
      name: {
        configurable: false,
        writable: false,

        value(v) {
          d.name = v;
          return this;
        }

      },
      handler: {
        configurable: false,
        writable: false,

        value(v) {
          d.handler = v;
          return this;
        }

      },
      multer: {
        configurable: false,
        writable: false,

        value(v) {
          d.options.multer = v;
          return this;
        }

      },
      endpoint: {
        configurable: false,
        writable: false,

        value(method, url) {
          d.endpoint = {
            method,
            url
          };
          return this;
        }

      },
      build: {
        configurable: false,
        writable: false,
        value: () => d
      }
    });
  };
}

const noop = v => v;

function handlerMaker(options = {}) {
  const {
    name,
    tableName,
    postprocessor = {},
    sanitizer = {}
  } = options;
  let {
    preprocessor = {}
  } = options;

  _odJs.ensure.nonEmptyString(name, 'handlerMaker requires name option.');

  _odJs.ensure.nonEmptyString(tableName, 'handlerMaker requires tableName option.'); // ensure(isSanitizer(sanitizer.id), 'handlerMaker requires sanitizer.id option.')
  // ensure(isSanitizer(sanitizer.ids), 'handlerMaker requires sanitizer.ids option.')
  // ensure(isSanitizer(sanitizer.add), 'handlerMaker requires sanitizer.add option.')
  // ensure(isSanitizer(sanitizer.get), 'handlerMaker requires sanitizer.get option.')
  // ensure(isSanitizer(sanitizer.modify), 'handlerMaker requires sanitizer.modify option.')


  if (_lodash.default.isFunction(preprocessor)) {
    preprocessor = {
      common: preprocessor
    };
  }

  const addPreprocessor = preprocessor.add || noop;
  const getPreprocessor = preprocessor.get || noop;
  const modifyPreprocessor = preprocessor.modify || noop;
  const removePreprocessor = preprocessor.remove || noop;
  const addPostprocessor = postprocessor.add || noop;
  const getPostprocessor = postprocessor.get || noop;
  const modifyPostprocessor = postprocessor.modify || noop;
  const removePostprocessor = postprocessor.remove || noop;

  async function getHandlerByHandlerMaker(context, id) {
    await (0, _bluebird.resolve)(preprocessor.common(context));
    await (0, _bluebird.resolve)(getPreprocessor(context)); // eslint-disable-next-line no-param-reassign

    id = sanitizer.id(id || context.getParam('id'));
    const conn = context.getMySQLConnection();
    const q = knex(tableName).select().where('id', id).toString();
    const [row] = await conn.query(q);
    await (0, _bluebird.resolve)(getPostprocessor(context, {
      row
    }));
    context.ensure(row, '해당 데이터를 찾을 수 없습니다.');
    return sanitizer.get(row);
  }

  async function addHandlerByHandlerMaker(context) {
    await (0, _bluebird.resolve)(preprocessor.common(context));
    await (0, _bluebird.resolve)(addPreprocessor(context));
    const conn = context.getMySQLConnection();
    const data = context.getParamObject(sanitizer.add);
    const sql = knex(tableName).insert(data).toString();

    try {
      const {
        insertId
      } = await conn.query(sql);
      await (0, _bluebird.resolve)(addPostprocessor(context, {
        data,
        id: insertId,
        op: 'add'
      }));
      return getHandlerByHandlerMaker(context, insertId);
    } catch (ex) {
      if ((0, _mysql.isForeignKeyError)(ex)) {
        context.ensure(false, `Foreign key error occurred while inserting ${name}`);
      }

      context.ensure(false, `Unknown error occurred while inserting ${name} - ${ex.message}`);
    }

    return undefined;
  }

  async function modifyHandlerByHandlerMaker(context, id, data) {
    await (0, _bluebird.resolve)(preprocessor.common(context));
    await (0, _bluebird.resolve)(modifyPreprocessor(context)); // eslint-disable-next-line no-param-reassign

    id = sanitizer.id(id || context.getParam('id')); // eslint-disable-next-line

    data = data ? sanitizer.modify(data) : context.getParamObject(sanitizer.modify);
    const conn = context.getMySQLConnection();
    const q = knex(tableName).update(data).where('id', id).toString();
    const {
      affectedRows
    } = await conn.query(q);
    context.ensure(affectedRows === 1, `Cannot find data of id [${id}]`);
    const dataReloaded = await getHandlerByHandlerMaker(context, id);
    await (0, _bluebird.resolve)(modifyPostprocessor(context, {
      data: dataReloaded,
      id,
      op: 'modify'
    }));
    return dataReloaded;
  }

  const idSanitizer = sane.anyOf(sane.array(sanitizer.id), sanitizer.id);

  const removeHandlerByHandlerMaker = async function removeHandlerByHandlerMaker(context, id) {
    await (0, _bluebird.resolve)(preprocessor.common(context));
    await (0, _bluebird.resolve)(removePreprocessor(context)); // eslint-disable-next-line no-param-reassign

    id = idSanitizer(id || context.getParam('id'));
    const conn = context.getMySQLConnection(); // 마음에 안들지만, 애초에 handler 를 od-node 에 넣은 것이 잘못된 선택이라고 생각하고 있음.
    // 그냥 진행한다. 2018-07-30 11:11

    let q;

    if (_lodash.default.isArray(id)) {
      q = knex(tableName).whereIn('id', id).select().toString();
    } else {
      q = knex(tableName).where('id', id).select().toString();
    }

    const data = await conn.query(q);

    if (_lodash.default.isArray(id)) {
      q = knex(tableName).whereIn('id', id).del().toString();
    } else {
      q = knex(tableName).where('id', id).del().toString();
    }

    const {
      affectedRows
    } = await conn.query(q);
    await (0, _bluebird.resolve)(removePostprocessor(context, {
      id,
      data,
      op: 'remove'
    }));
    return {
      num: affectedRows
    };
  };

  return {
    add: addHandlerByHandlerMaker,
    get: getHandlerByHandlerMaker,
    modify: modifyHandlerByHandlerMaker,
    remove: removeHandlerByHandlerMaker,
    route: (app, prefix) => {
      if (!prefix.endsWith('/')) {
        // eslint-disable-next-line no-param-reassign
        prefix += '/';
      }

      app.defineHandler(`add${name}`, d => d.handler(addHandlerByHandlerMaker).endpoint('post', `${prefix}`));
      app.defineHandler(`get${name}`, d => d.handler(getHandlerByHandlerMaker).endpoint('get', `${prefix}:id`));
      app.defineHandler(`modify${name}`, d => d.handler(modifyHandlerByHandlerMaker).endpoint('post', `${prefix}:id`));
      app.defineHandler(`remove${name}`, d => d.handler(removeHandlerByHandlerMaker).endpoint('delete', `${prefix}:id`));
      app.defineHandler(`removeMulti${name}`, d => d.handler(removeHandlerByHandlerMaker).endpoint('delete', `${prefix}`));
    }
  };
}