"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createLocalStorageDriver;

var _bluebird = _interopRequireDefault(require("bluebird"));

var _debug = _interopRequireDefault(require("debug"));

var _lodash = _interopRequireDefault(require("lodash"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _odJs = require("od-js");

var _path = _interopRequireDefault(require("path"));

var _v = _interopRequireDefault(require("uuid/v4"));

var _utils = require("../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-underscore-dangle */
const driverPrototype = {
  async init() {
    const storagePath = _path.default.join(this._basePath, this._name);

    this._debug(`Creating storage path : ${storagePath}`);

    await new _bluebird.default((resolve, reject) => {
      (0, _mkdirp.default)(storagePath, err => err ? reject(err) : resolve());
    });
  },

  /**
   * 이미지를 저장하고 생성된 이미지 아이디를 반환한다.
   *
   * @param obj {object|string|array} 이미지 경로 또는 multer 를 통하여 받아진 객체
   * @param [options] {object}
   * @return {Promise.<string>}
   */
  async save(obj, options = {}) {
    let imagePath;

    if (_lodash.default.isString(obj)) {
      // path
      imagePath = obj;
    } else if (_lodash.default.isArray(obj)) {
      return this.save(obj[0]);
    } else if (_lodash.default.isObject(obj)) {
      imagePath = obj.path;
    } // const { contentType = 'image/jpeg', ext = '.jpg' } = options


    const {
      originalname,
      removeOriginal = this._removeOriginal
    } = options;
    let {
      ext
    } = options;
    ext = ext || (originalname ? _path.default.extname(originalname) : null) || '.jpg';

    this._ensure.nonEmptyString(imagePath, this._paramError('Invalid image path'));

    const imageId = (0, _v.default)();

    const finalImagePath = _path.default.join(this._basePath, this._name, `${imageId}${ext}`);

    if (removeOriginal) {
      await (0, _utils.moveFile)(imagePath, finalImagePath);
    } else {
      await (0, _utils.copyFile)(imagePath, finalImagePath);
    }

    return imageId;
  },

  async remove(imageId, {
    ext = '.jpg'
  } = {}) {
    const finalImagePath = _path.default.join(this._basePath, this._name, `${imageId}${ext}`);

    return (0, _utils.rmrf)(finalImagePath);
  }

};

function createLocalStorageDriver(options, definition) {
  // TODO: sanitize options
  const {
    path,
    removeOriginal = true
  } = options;
  const {
    name
  } = definition;
  return Object.create(driverPrototype, {
    // TODO: how to pass along _ensure, _paramError, _sanitizer, ....
    _ensure: {
      configurable: false,
      writable: false,
      value: _odJs.ensure
    },
    _paramError: {
      configurable: false,
      writable: false,
      value: name => {
        return {
          error: ({
            value
          }) => `Invalid [${name}] - ${value} / ${typeof value}`
        };
      }
    },
    _debug: {
      configurable: false,
      writable: false,
      value: (0, _debug.default)(`od:localStorage:${name}`)
    },
    _name: {
      configurable: false,
      writable: false,
      value: name
    },
    _basePath: {
      configurable: false,
      writable: false,
      value: path
    },
    _removeOriginal: {
      configurable: false,
      writable: false,
      value: removeOriginal
    }
  });
}