"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createLocalStorageDriver;

var _bluebird = _interopRequireWildcard(require("bluebird"));

var _awsSdk = _interopRequireDefault(require("aws-sdk"));

var _debug = _interopRequireDefault(require("debug"));

var _fs = _interopRequireDefault(require("fs"));

var _odJs = require("od-js");

var _path = _interopRequireDefault(require("path"));

var _v = _interopRequireDefault(require("uuid/v4"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

/* eslint-disable no-underscore-dangle */
// http://docs.aws.amazon.com/cli/latest/userguide/cli-environment.html
//
// AWS_ACCESS_KEY_ID
// AWS_SECRET_ACCESS_KEY
// AWS_DEFAULT_REGION
const driverPrototype = {
  async init() {
    _awsSdk.default.config.loadFromPath(this._awsCredFilePath); // const storagePath = path.join(this._basePath, this._name)
    //
    // this._debug(`Creating storage path : ${storagePath}`)
    // await new Promise((resolve, reject) => {
    //   mkdirp(storagePath, (err) => err ? reject(err) : resolve())
    // })

  },

  /**
   * 이미지를 저장하고 생성된 이미지 아이디를 반환한다.
   *
   * @param obj {object|string|array} 이미지 경로 또는 multer 를 통하여 받아진 객체
   * @param [options] {object}
   * @return {Promise.<string>}
   */
  async save(obj, options = {}) {
    let imagePath;

    if (_odJs._.isString(obj)) {
      // path
      imagePath = obj;
    } else if (_odJs._.isArray(obj)) {
      return this.save(obj[0]);
    } else if (_odJs._.isObject(obj)) {
      imagePath = obj.path;
    }

    let {
      fileName,
      contentType,
      ext
    } = options;
    const {
      mimetype,
      originalname,
      asAttachment,
      uploadOptions: uo = {},
      removeOriginal = this._removeOriginal
    } = options;
    contentType = contentType || mimetype || 'image/jpeg';
    ext = ext || (originalname ? _path.default.extname(originalname) : null) || '.jpg';
    fileName = fileName || originalname;

    this._ensure.nonEmptyString(imagePath, this._paramError('Invalid image path'));

    const imageId = (0, _v.default)();
    const dnName = fileName || `${imageId}${ext}`;
    const s3 = new _awsSdk.default.S3();
    const bucketName = this._bucketName;
    const key = `${this._prefix}${imageId}${ext}`;
    const uploadOptions = {
      Bucket: bucketName,
      ContentType: contentType,
      Key: key,
      ACL: 'public-read',
      Body: _fs.default.createReadStream(imagePath)
    };

    if (asAttachment) {
      uploadOptions.ContentDisposition = `attachment; filename="${encodeURIComponent(dnName)}"`;
    } else {
      uploadOptions.ContentDisposition = `inline; filename="${encodeURIComponent(dnName)}"`;
    }

    await new _bluebird.default((resolve, reject) => {
      const s3UploadOptions = Object.assign(uploadOptions, uo);
      s3.putObject(s3UploadOptions, (err, data) => {
        err && reject(err);
        !err && resolve(data);
      });
    });

    if (removeOriginal) {
      await new _bluebird.default(resolve => {
        _fs.default.unlink(imagePath, resolve);
      });
    }

    return imageId;
  },

  async listBucket(maxSize = 1000) {
    return new _bluebird.default((resolve, reject) => {
      const s3 = new _awsSdk.default.S3();
      s3.listObjects({
        Bucket: this._bucketName,
        MaxKeys: maxSize
      }, (err, data) => {
        err && reject(err);
        !err && resolve(data);
      });
    });
  },

  async emptyBucket() {
    const data = await this.listBucket();

    if (data.Contents.length === 0) {
      return (0, _bluebird.resolve)();
    }

    const param = {
      Bucket: this._bucketName,
      Delete: {
        Objects: data.Contents.map(c => {
          return {
            Key: c.Key
          };
        }),
        Quiet: true
      }
    };
    return new _bluebird.default((resolve, reject) => {
      const s3 = new _awsSdk.default.S3();
      s3.deleteObjects(param, (err, data) => {
        err && reject(err);
        !err && resolve(data);
      });
    });
  },

  get bucketName() {
    return this._bucketName;
  },

  async remove(imageId, {
    ext = '.jpg'
  } = {}) {
    const s3 = new _awsSdk.default.S3();
    const bucketName = this._bucketName;
    const key = `${this._prefix}${imageId}${ext}`;
    return new _bluebird.default((resolve, reject) => {
      s3.deleteObject({
        Bucket: bucketName,
        Key: key
      }, (err, data) => {
        err && reject(err);
        !err && resolve(data);
      });
    });
  }

};

function createLocalStorageDriver(options, definition) {
  // TODO: sanitize options
  const {
    prefix = '',
    credential,
    bucket,
    removeOriginal = true
  } = options;
  const {
    name
  } = definition;
  return Object.create(driverPrototype, {
    _ensure: {
      configurable: false,
      writable: false,
      value: _odJs.ensure
    },
    _paramError: {
      configurable: false,
      writable: false,
      value: name => {
        return {
          error: ({
            value
          }) => `Invalid [${name}] - ${value} / ${typeof value}`
        };
      }
    },
    _debug: {
      configurable: false,
      writable: false,
      value: (0, _debug.default)(`od:localStorage:${name}`)
    },
    _name: {
      configurable: false,
      writable: false,
      value: name
    },
    _removeOriginal: {
      configurable: false,
      writable: false,
      value: removeOriginal
    },
    _bucketName: {
      configurable: false,
      writable: false,
      value: bucket
    },
    _prefix: {
      // 해당 버킷내의 경로 prefix (디렉토리)
      configurable: false,
      writable: false,
      value: prefix
    },
    _awsCredFilePath: {
      // aws access key, secret key 등을 담고 있는 json 파일의 경로
      configurable: false,
      writable: false,
      value: credential
    }
  });
}